# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from base64 import b64encode
import unittest
from zlib import compress

from mixbox.vendor import six
from mixbox.vendor.six import u

from cybox.objects.artifact_object import (Artifact, Base64Encoding,
        Bz2Compression, RawArtifact, XOREncryption, ZlibCompression)
from cybox.test import round_trip
from cybox.test.objects import ObjectTestCase


class TestRawArtifact(unittest.TestCase):

    def test_xml_output(self):
        # A RawArtifact stores a Unicode string, even though it typically
        # consists only of valid Base64 characters.
        data = u("0123456789abcdef")
        ra = RawArtifact(data)

        expected_data = data.encode('utf-8')
        self.assertTrue(expected_data in ra.to_xml())


class TestArtifactEncoding(unittest.TestCase):

    def test_cannot_create_artifact_from_unicode_data(self):
        self.assertRaises(ValueError, Artifact, u("abc123"))

    def test_setting_ascii_artifact_data_no_packaging(self):
        a = Artifact()
        a.data = b"abc123"
        self.assertEqual(six.binary_type, type(a.data))
        self.assertEqual(six.text_type, type(a.packed_data))

    def test_cannot_set_nonascii_data_with_no_packaging(self):
        a = Artifact()
        # You can set this data, but if you don't add any packaging, you should
        # get an error when trying to get the packed data, since it can't be
        # encoded as ASCII.
        a.data = b"\x00abc123\xff"
        self.assertEqual(six.binary_type, type(a.data))
        self.assertRaises(ValueError, _get_packed_data, a)

        # With Base64 encoding, we can retrieve this.
        a.packaging.append(Base64Encoding())
        self.assertEqual("AGFiYzEyM/8=", a.packed_data)

    def test_setting_ascii_artifact_packed_data_no_packaging(self):
        a = Artifact()
        a.packed_data = u("abc123")
        self.assertEqual(six.binary_type, type(a.data))
        self.assertEqual(six.text_type, type(a.packed_data))

    def test_cannot_set_nonascii_artifact_packed_data(self):
        a = Artifact()
        a.packed_data = u("\x00abc123\xff")
        self.assertEqual(six.text_type, type(a.packed_data))

        #TODO: Should this raise an error sooner, since there's nothing we can
        # do at this point? There's no reason that the packed_data should
        # contain non-ascii characters.
        self.assertRaises(UnicodeEncodeError, _get_data, a)


class TestArtifact(ObjectTestCase, unittest.TestCase):
    object_type = "ArtifactObjectType"
    klass = Artifact

    ascii_data = b"ABCDEFGHIJKLMNOPQRSTUVWZYZ0123456879"
    binary_data = b"\xde\xad\xbe\xef Dead Beef"

    # The raw_artifact data in a JSON/dict representation should always be
    # ASCII byte data (typically Base64-encoded, but this is not required).
    _full_dict = {
        'raw_artifact': "Here is a blob of text.",
        'type': Artifact.TYPE_NETWORK,
        'xsi:type': object_type,
    }

    def test_set_data_and_packed_data(self):
        a = Artifact()
        self.assertEqual(a.data, None)
        self.assertEqual(a.packed_data, None)

        a.data = b"Blob"
        self.assertRaises(ValueError, _set_packed_data, a, u("blob"))
        a.data = None

        a.packed_data = u("Blob")
        self.assertRaises(ValueError, _set_data, a, b"blob")
        a.packed_data = None

    def test_round_trip(self):
        # Without any packaging, the only data an Artifact can encode
        # successfully is ASCII data.
        a = Artifact(self.ascii_data, Artifact.TYPE_GENERIC)
        a2 = round_trip(a, Artifact)
        self.assertEqual(a.to_dict(), a2.to_dict())

    def test_non_ascii_round_trip_raises_error(self):
        a = Artifact(self.binary_data, Artifact.TYPE_GENERIC)
        # Since the data is non-ASCII, this should raise an error.
        self.assertRaises(ValueError, round_trip, a, Artifact)

    def test_base64_encoding(self):
        a = Artifact(self.binary_data)
        a.packaging.append(Base64Encoding())
        a2 = round_trip(a, Artifact)
        self.assertEqual(self.binary_data, a2.data)

        expected = b64encode(self.binary_data).decode('ascii')
        self.assertEqual(expected, a2.packed_data)

    def test_zlib_base64_encoding(self):
        a = Artifact(self.binary_data)
        a.packaging.append(ZlibCompression())
        a.packaging.append(Base64Encoding())
        a2 = round_trip(a, Artifact)
        self.assertEqual(self.binary_data, a2.data)

        expected = b64encode(compress(self.binary_data)).decode('ascii')
        self.assertEqual(expected, a2.packed_data)

    def test_encryption(self):
        a = Artifact(self.binary_data)
        a.packaging.append(XOREncryption(0x4a))
        a.packaging.append(Base64Encoding())
        a2 = round_trip(a, Artifact)

        self.assertEqual(self.binary_data, a2.data)


def _get_data(artifact):
    return artifact.data


def _set_data(artifact, data):
    artifact.data = data


def _get_packed_data(artifact):
    return artifact.packed_data


def _set_packed_data(artifact, packed_data):
    artifact.packed_data = packed_data


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_executable_file_object as win_executable_file_binding
from cybox.common import (DateTime, DigitalSignature, Float, HashList,
        HexBinary, Integer, Long, NonNegativeInteger, String, UnsignedLong, PositiveInteger)
from cybox.objects.win_file_object import WinFile


class Entropy(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.EntropyType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    value = fields.TypedField("Value", Float)
    min = fields.TypedField("Min", Float)
    max = fields.TypedField("Max", Float)


class PEBuildInformation(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEBuildInformationType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    linker_name = fields.TypedField("Linker_Name", String)
    linker_version = fields.TypedField("Linker_Version", String)
    compiler_name = fields.TypedField("Compiler_Name", String)
    compiler_version = fields.TypedField("Compiler_Version", String)


class PEExportedFunction(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEExportedFunctionType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    function_name = fields.TypedField("Function_Name", String)
    entry_point = fields.TypedField("Entry_Point", HexBinary)
    ordinal = fields.TypedField("Ordinal", NonNegativeInteger)


class PEExportedFunctions(entities.EntityList):
    _binding_class = win_executable_file_binding.PEExportedFunctionsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"
    exported_function = fields.TypedField("Exported_Function", PEExportedFunction, multiple=True)


class PEExports(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEExportsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    name = fields.TypedField("Name", String)
    exported_functions = fields.TypedField("Exported_Functions", PEExportedFunctions)
    number_of_functions = fields.TypedField("Number_Of_Functions", Integer)
    exports_time_stamp = fields.TypedField("Exports_Time_Stamp", DateTime)
    number_of_addresses = fields.TypedField("Number_Of_Addresses", Long)
    number_of_names = fields.TypedField("Number_Of_Names", Long)


class DOSHeader(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.DOSHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    e_magic = fields.TypedField("e_magic", HexBinary)
    e_cblp = fields.TypedField("e_cblp", HexBinary)
    e_cp = fields.TypedField("e_cp", HexBinary)
    e_crlc = fields.TypedField("e_crlc", HexBinary)
    e_cparhdr = fields.TypedField("e_cparhdr", HexBinary)
    e_minalloc = fields.TypedField("e_minalloc", HexBinary)
    e_maxalloc = fields.TypedField("e_maxalloc", HexBinary)
    e_ss = fields.TypedField("e_ss", HexBinary)
    e_sp = fields.TypedField("e_sp", HexBinary)
    e_csum = fields.TypedField("e_csum", HexBinary)
    e_ip = fields.TypedField("e_ip", HexBinary)
    e_cs = fields.TypedField("e_cs", HexBinary)
    e_lfarlc = fields.TypedField("e_lfarlc", HexBinary)
    e_ovro = fields.TypedField("e_ovro", HexBinary)
    e_oemid = fields.TypedField("e_oemid", HexBinary)
    e_oeminfo = fields.TypedField("e_oeminfo", HexBinary)
    reserved2 = fields.TypedField("reserved2", HexBinary)
    e_lfanew = fields.TypedField("e_lfanew", HexBinary)
    hashes = fields.TypedField("Hashes", HashList)
    #reserved1 = [] unsupported for now


class PEFileHeader(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEFileHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    machine = fields.TypedField("Machine", HexBinary)
    number_of_sections = fields.TypedField("Number_Of_Sections", NonNegativeInteger)
    time_date_stamp = fields.TypedField("Time_Date_Stamp", HexBinary)
    pointer_to_symbol_table = fields.TypedField("Pointer_To_Symbol_Table", HexBinary)
    number_of_symbols = fields.TypedField("Number_Of_Symbols", NonNegativeInteger)
    size_of_optional_header = fields.TypedField("Size_Of_Optional_Header", HexBinary)
    characteristics = fields.TypedField("Characteristics", HexBinary)
    hashes = fields.TypedField("Hashes", HashList)


class PEDataDirectoryStruct(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEDataDirectoryStructType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    virtual_address = fields.TypedField("Virtual_Address", HexBinary)
    size = fields.TypedField("Size", NonNegativeInteger)


class DataDirectory(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.DataDirectoryType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    export_table = fields.TypedField("Export_Table", PEDataDirectoryStruct)
    import_table = fields.TypedField("Import_Table", PEDataDirectoryStruct)
    resource_table = fields.TypedField("Resource_Table", PEDataDirectoryStruct)
    exception_table = fields.TypedField("Exception_Table", PEDataDirectoryStruct)
    certificate_table = fields.TypedField("Certificate_Table", PEDataDirectoryStruct)
    base_relocation_table = fields.TypedField("Base_Relocation_Table", PEDataDirectoryStruct)
    debug = fields.TypedField("Debug", PEDataDirectoryStruct)
    architecture = fields.TypedField("Architecture", PEDataDirectoryStruct)
    global_ptr = fields.TypedField("Global_Ptr", PEDataDirectoryStruct)
    tls_table = fields.TypedField("Tls_Table", PEDataDirectoryStruct)
    load_config_table = fields.TypedField("Load_Config_Table", PEDataDirectoryStruct)
    bound_import = fields.TypedField("Bound_Import", PEDataDirectoryStruct)
    import_address_table = fields.TypedField("Import_Address_Table", PEDataDirectoryStruct)
    delay_import_descriptor = fields.TypedField("Delay_Import_Descriptor", PEDataDirectoryStruct)
    clr_runtime_header = fields.TypedField("CLR_Runtime_Header", PEDataDirectoryStruct)
    reserved = fields.TypedField("Reserved", PEDataDirectoryStruct)


class PEOptionalHeader(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEOptionalHeaderType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    magic = fields.TypedField("Magic", HexBinary)
    major_linker_version = fields.TypedField("Major_Linker_Version", HexBinary)
    minor_linker_version = fields.TypedField("Minor_Linker_Version", HexBinary)
    size_of_code = fields.TypedField("Size_Of_Code", HexBinary)
    size_of_initialized_data = fields.TypedField("Size_Of_Initialized_Data", HexBinary)
    size_of_uninitialized_data = fields.TypedField("Size_Of_Uninitialized_Data", HexBinary)
    address_of_entry_point = fields.TypedField("Address_Of_Entry_Point", HexBinary)
    base_of_code = fields.TypedField("Base_Of_Code", HexBinary)
    base_of_data = fields.TypedField("Base_Of_Data", HexBinary)
    image_base = fields.TypedField("Image_Base", HexBinary)
    section_alignment = fields.TypedField("Section_Alignment", HexBinary)
    file_alignment = fields.TypedField("File_Alignment", HexBinary)
    major_os_version = fields.TypedField("Major_OS_Version", HexBinary)
    minor_os_version = fields.TypedField("Minor_OS_Version", HexBinary)
    major_image_version = fields.TypedField("Major_Image_Version", HexBinary)
    minor_image_version = fields.TypedField("Minor_Image_Version", HexBinary)
    major_subsystem_version = fields.TypedField("Major_Subsystem_Version", HexBinary)
    minor_subsystem_version = fields.TypedField("Minor_Subsystem_Version", HexBinary)
    win32_version_value = fields.TypedField("Win32_Version_Value", HexBinary)
    size_of_image = fields.TypedField("Size_Of_Image", HexBinary)
    size_of_headers = fields.TypedField("Size_Of_Headers", HexBinary)
    checksum = fields.TypedField("Checksum", HexBinary)
    subsystem = fields.TypedField("Subsystem", HexBinary)
    dll_characteristics = fields.TypedField("DLL_Characteristics", HexBinary)
    size_of_stack_reserve = fields.TypedField("Size_Of_Stack_Reserve", HexBinary)
    size_of_stack_commit = fields.TypedField("Size_Of_Stack_Commit", HexBinary)
    size_of_heap_reserve = fields.TypedField("Size_Of_Heap_Reserve", HexBinary)
    size_of_heap_commit = fields.TypedField("Size_Of_Heap_Commit", HexBinary)
    loader_flags = fields.TypedField("Loader_Flags", HexBinary)
    number_of_rva_and_sizes = fields.TypedField("Number_Of_Rva_And_Sizes", HexBinary)
    data_directory = fields.TypedField("Data_Directory", DataDirectory)
    hashes = fields.TypedField("Hashes", HashList)


class PEHeaders(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEHeadersType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    dos_header = fields.TypedField("DOS_Header", DOSHeader)
    signature = fields.TypedField("Signature", HexBinary)
    file_header = fields.TypedField("File_Header", PEFileHeader)
    optional_header = fields.TypedField("Optional_Header", PEOptionalHeader)
    entropy = fields.TypedField("Entropy", Entropy)
    hashes = fields.TypedField("Hashes", HashList)


class PEImportedFunction(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEImportedFunctionType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    function_name = fields.TypedField("Function_Name", String)
    hint = fields.TypedField("Hint", HexBinary)
    ordinal = fields.TypedField("Ordinal", NonNegativeInteger)
    bound = fields.TypedField("Bound", HexBinary)
    virtual_address = fields.TypedField("Virtual_Address", HexBinary)


class PEImportedFunctions(entities.EntityList):
    _binding_class = win_executable_file_binding.PEImportedFunctionsType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"
    imported_function = fields.TypedField("Imported_Function", PEImportedFunction, multiple=True)


class PEImport(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEImportType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    delay_load = fields.TypedField("delay_load")
    initially_visible = fields.TypedField("initially_visible")
    file_name = fields.TypedField("File_Name", String)
    imported_functions = fields.TypedField("Imported_Functions", PEImportedFunctions)
    virtual_address = fields.TypedField("Virtual_Address", HexBinary)


class PEImportList(entities.EntityList):
    _binding_class = win_executable_file_binding.PEImportListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"
    import_ = fields.TypedField("Import", PEImport, multiple=True)


class PEChecksum(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEChecksumType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    pe_computed_api = fields.TypedField("PE_Computed_API", Long)
    pe_file_api = fields.TypedField("PE_File_API", Long)
    pe_file_raw = fields.TypedField("PE_File_Raw", Long)


class PEResource(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEResourceType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    type_ = fields.TypedField("Type", String)
    name = fields.TypedField("Name", String)
    size = fields.TypedField("Size", PositiveInteger)
    virtual_address = fields.TypedField("Virtual_Address", HexBinary)
    language = fields.TypedField("Language", String)
    sub_language = fields.TypedField("Sub_Language", String)
    hashes = fields.TypedField("Hashes", HashList)
    data = fields.TypedField("Data", String)


class PEResourceList(entities.EntityList):
    _binding_class = win_executable_file_binding.PEResourceListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"
    resource = fields.TypedField("Resource", PEResource, multiple=True)

    #VersionInfoResource temporary fix
    @classmethod
    def from_list(cls, seq):
        if not seq:
            return None

        # TODO (bworrell): Should this just call cls(). Does this class need
        # an EntityFactory?
        pe_resource_list_ = super(PEResourceList, cls).from_list(seq)

        for pe_resource_dict in seq:
            if PEVersionInfoResource.keyword_test(pe_resource_dict):
                pe_resource_list_.append(PEVersionInfoResource.from_dict(pe_resource_dict))
            else:
                pe_resource_list_.append(PEResource.from_dict(pe_resource_dict))
        return pe_resource_list_


class PESectionHeaderStruct(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PESectionHeaderStructType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    name = fields.TypedField("Name", String)
    virtual_size = fields.TypedField("Virtual_Size", HexBinary)
    virtual_address = fields.TypedField("Virtual_Address", HexBinary)
    size_of_raw_data = fields.TypedField("Size_Of_Raw_Data", HexBinary)
    pointer_to_raw_data = fields.TypedField("Pointer_To_Raw_Data", HexBinary)
    pointer_to_relocations = fields.TypedField("Pointer_To_Relocations", HexBinary)
    pointer_to_linenumbers = fields.TypedField("Pointer_To_Linenumbers", HexBinary)
    number_of_relocations = fields.TypedField("Number_Of_Relocations", NonNegativeInteger)
    number_of_linenumbers = fields.TypedField("Number_Of_Linenumbers", NonNegativeInteger)
    characteristics = fields.TypedField("Characteristics", HexBinary)


class PESection(entities.Entity):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PESectionType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    section_header = fields.TypedField("Section_Header", PESectionHeaderStruct)
    data_hashes = fields.TypedField("Data_Hashes", HashList)
    entropy = fields.TypedField("Entropy", Entropy)
    header_hashes = fields.TypedField("Header_Hashes", HashList)


class PESectionList(entities.EntityList):
    _binding_class = win_executable_file_binding.PESectionListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"
    section = fields.TypedField("Section", PESection, multiple=True)


class PEVersionInfoResource(PEResource):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.PEVersionInfoResourceType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"

    comments = fields.TypedField("Comments", String)
    companyname = fields.TypedField("CompanyName", String)
    filedescription = fields.TypedField("FileDescription", String)
    fileversion = fields.TypedField("FileVersion", String)
    internalname = fields.TypedField("InternalName", String)
    langid = fields.TypedField("LangID", String)
    legalcopyright = fields.TypedField("LegalCopyright", String)
    legaltrademarks = fields.TypedField("LegalTrademarks", String)
    originalfilename = fields.TypedField("OriginalFilename", String)
    privatebuild = fields.TypedField("PrivateBuild", String)
    productname = fields.TypedField("ProductName", String)
    productversion = fields.TypedField("ProductVersion", String)
    specialbuild = fields.TypedField("SpecialBuild", String)

    @staticmethod
    def keyword_test(pe_resource_dict):
        keywords_list = ['comments',
                         'companyname',
                         'filedescription',
                         'fileversion',
                         'internalname',
                         'langid',
                         'legalcopyright',
                         'originalfilename',
                         'privatebuild',
                         'productname',
                         'productversion',
                         'specialbuild']

        return any(key in keywords_list for key in pe_resource_dict)


class WinExecutableFile(WinFile):
    _binding = win_executable_file_binding
    _binding_class = win_executable_file_binding.WindowsExecutableFileObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-executable-file-2"
    _XSI_NS = "WinExecutableFileObj"
    _XSI_TYPE = "WindowsExecutableFileObjectType"

    build_information = fields.TypedField("Build_Information", PEBuildInformation)
    digital_signature = fields.TypedField("Digital_Signature", DigitalSignature)
    exports = fields.TypedField("Exports", PEExports)
    extraneous_bytes = fields.TypedField("Extraneous_Bytes", Integer)
    headers = fields.TypedField("Headers", PEHeaders)
    imports = fields.TypedField("Imports", PEImportList)
    pe_checksum = fields.TypedField("PE_Checksum", PEChecksum)
    resources = fields.TypedField("Resources", PEResourceList)
    sections = fields.TypedField("Sections", PESectionList)
    type_ = fields.TypedField("Type", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.gui_window_object as gui_window_binding
from cybox.objects.gui_object import GUI
from cybox.common import String


class GUIWindow(GUI):
    _binding = gui_window_binding
    _binding_class = gui_window_binding.GUIWindowObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/gui-window-2"
    _XSI_NS = "GUIWindowObj"
    _XSI_TYPE = "GUIWindowObjectType"

    owner_window = fields.TypedField("Owner_Window", String)
    parent_window = fields.TypedField("Parent_Window", String)
    window_display_name = fields.TypedField("Window_Display_Name", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import address_object
from . import uri_object


class DNSRecordObjectType(cybox_common.ObjectPropertiesType):
    """The DNSRecordObjectType type is intended to characterize an
    individual DNS record."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, Description=None, Queried_Date=None, Domain_Name=None, IP_Address=None, Address_Class=None, Entry_Type=None, Record_Name=None, Record_Type=None, TTL=None, Flags=None, Data_Length=None, Record_Data=None):
        super(DNSRecordObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.Description = Description
        self.Queried_Date = Queried_Date
        self.Domain_Name = Domain_Name
        self.IP_Address = IP_Address
        self.Address_Class = Address_Class
        self.Entry_Type = Entry_Type
        self.Record_Name = Record_Name
        self.Record_Type = Record_Type
        self.TTL = TTL
        self.Flags = Flags
        self.Data_Length = Data_Length
        self.Record_Data = Record_Data
    def factory(*args_, **kwargs_):
        if DNSRecordObjectType.subclass:
            return DNSRecordObjectType.subclass(*args_, **kwargs_)
        else:
            return DNSRecordObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Description(self): return self.Description
    def set_Description(self, Description): self.Description = Description
    def get_Queried_Date(self): return self.Queried_Date
    def set_Queried_Date(self, Queried_Date): self.Queried_Date = Queried_Date
    def get_Domain_Name(self): return self.Domain_Name
    def set_Domain_Name(self, Domain_Name): self.Domain_Name = Domain_Name
    def get_IP_Address(self): return self.IP_Address
    def set_IP_Address(self, IP_Address): self.IP_Address = IP_Address
    def get_Address_Class(self): return self.Address_Class
    def set_Address_Class(self, Address_Class): self.Address_Class = Address_Class
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Entry_Type(self): return self.Entry_Type
    def set_Entry_Type(self, Entry_Type): self.Entry_Type = Entry_Type
    def get_Record_Name(self): return self.Record_Name
    def set_Record_Name(self, Record_Name): self.Record_Name = Record_Name
    def get_Record_Type(self): return self.Record_Type
    def set_Record_Type(self, Record_Type): self.Record_Type = Record_Type
    def get_TTL(self): return self.TTL
    def set_TTL(self, TTL): self.TTL = TTL
    def validate_IntegerObjectPropertyType(self, value):
        # Validate type cybox_common.IntegerObjectPropertyType, a restriction on None.
        pass
    def get_Flags(self): return self.Flags
    def set_Flags(self, Flags): self.Flags = Flags
    def validate_HexBinaryObjectPropertyType(self, value):
        # Validate type cybox_common.HexBinaryObjectPropertyType, a restriction on None.
        pass
    def get_Data_Length(self): return self.Data_Length
    def set_Data_Length(self, Data_Length): self.Data_Length = Data_Length
    def get_Record_Data(self): return self.Record_Data
    def set_Record_Data(self, Record_Data): self.Record_Data = Record_Data
    def hasContent_(self):
        if (
            self.Description is not None or
            self.Queried_Date is not None or
            self.Domain_Name is not None or
            self.IP_Address is not None or
            self.Address_Class is not None or
            self.Entry_Type is not None or
            self.Record_Name is not None or
            self.Record_Type is not None or
            self.TTL is not None or
            self.Flags is not None or
            self.Data_Length is not None or
            self.Record_Data is not None or
            super(DNSRecordObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='DNSRecordObj:', name_='DNSRecordObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='DNSRecordObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='DNSRecordObj:', name_='DNSRecordObjectType'):
        super(DNSRecordObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='DNSRecordObjectType')
    def exportChildren(self, lwrite, level, namespace_='DNSRecordObj:', name_='DNSRecordObjectType', fromsubclass_=False, pretty_print=True):
        super(DNSRecordObjectType, self).exportChildren(lwrite, level, 'DNSRecordObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Description is not None:
            self.Description.export(lwrite, level, 'DNSRecordObj:', name_='Description', pretty_print=pretty_print)
        if self.Queried_Date is not None:
            self.Queried_Date.export(lwrite, level, 'DNSRecordObj:', name_='Queried_Date', pretty_print=pretty_print)
        if self.Domain_Name is not None:
            self.Domain_Name.export(lwrite, level, 'DNSRecordObj:', name_='Domain_Name', pretty_print=pretty_print)
        if self.IP_Address is not None:
            self.IP_Address.export(lwrite, level, 'DNSRecordObj:', name_='IP_Address', pretty_print=pretty_print)
        if self.Address_Class is not None:
            self.Address_Class.export(lwrite, level, 'DNSRecordObj:', name_='Address_Class', pretty_print=pretty_print)
        if self.Entry_Type is not None:
            self.Entry_Type.export(lwrite, level, 'DNSRecordObj:', name_='Entry_Type', pretty_print=pretty_print)
        if self.Record_Name is not None:
            self.Record_Name.export(lwrite, level, 'DNSRecordObj:', name_='Record_Name', pretty_print=pretty_print)
        if self.Record_Type is not None:
            self.Record_Type.export(lwrite, level, 'DNSRecordObj:', name_='Record_Type', pretty_print=pretty_print)
        if self.TTL is not None:
            self.TTL.export(lwrite, level, 'DNSRecordObj:', name_='TTL', pretty_print=pretty_print)
        if self.Flags is not None:
            self.Flags.export(lwrite, level, 'DNSRecordObj:', name_='Flags', pretty_print=pretty_print)
        if self.Data_Length is not None:
            self.Data_Length.export(lwrite, level, 'DNSRecordObj:', name_='Data_Length', pretty_print=pretty_print)
        if self.Record_Data is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite('<%sRecord_Data>%s</%sRecord_Data>%s' % ('DNSRecordObj:', self.gds_format_string(quote_xml(self.Record_Data), input_name='Record_Data'), 'DNSRecordObj:', eol_))
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(DNSRecordObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Description':
            obj_ = cybox_common.StructuredTextType.factory()
            obj_.build(child_)
            self.set_Description(obj_)
        elif nodeName_ == 'Queried_Date':
            obj_ = cybox_common.DateTimeObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Queried_Date(obj_)
        elif nodeName_ == 'Domain_Name':
            obj_ = uri_object.URIObjectType.factory()
            obj_.build(child_)
            self.set_Domain_Name(obj_)
        elif nodeName_ == 'IP_Address':
            obj_ = address_object.AddressObjectType.factory()
            obj_.build(child_)
            self.set_IP_Address(obj_)
        elif nodeName_ == 'Address_Class':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Address_Class(obj_)
        elif nodeName_ == 'Entry_Type':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Entry_Type(obj_)
        elif nodeName_ == 'Record_Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Record_Name(obj_)
        elif nodeName_ == 'Record_Type':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Record_Type(obj_)
        elif nodeName_ == 'TTL':
            obj_ = cybox_common.IntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_TTL(obj_)
        elif nodeName_ == 'Flags':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Flags(obj_)
        elif nodeName_ == 'Data_Length':
            obj_ = cybox_common.IntegerObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Data_Length(obj_)
        elif nodeName_ == 'Record_Data':
            Record_Data_ = child_.text
            Record_Data_ = self.gds_validate_string(Record_Data_, node, 'Record_Data')
            self.Record_Data = Record_Data_
        super(DNSRecordObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class DNSRecordObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Entry_Type': cybox_common.StringObjectPropertyType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'SubDatum': cybox_common.MetadataType,
    'Record_Name': cybox_common.StringObjectPropertyType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'URI': uri_object.URIObjectType,
    'Value': cybox_common.AnyURIObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'TTL': cybox_common.IntegerObjectPropertyType,
    'IP_Address': address_object.AddressObjectType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Language': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'Address_Class': cybox_common.StringObjectPropertyType,
    'Contributors': cybox_common.PersonnelType,
    'Reference_Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Record_Type': cybox_common.StringObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Domain_Name': uri_object.URIObjectType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'VLAN_Name': cybox_common.StringObjectPropertyType,
    'Address': address_object.AddressObjectType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Address_Value': cybox_common.StringObjectPropertyType,
    'VLAN_Num': cybox_common.IntegerObjectPropertyType,
    'Flags': cybox_common.HexBinaryObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Data_Length': cybox_common.IntegerObjectPropertyType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Tool': cybox_common.ToolInformationType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'DNS_Record'
        rootClass = DNSRecordObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'DNS_Record'
        rootClass = DNSRecordObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'DNS_Record'
        rootClass = DNSRecordObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="DNS_Record",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "DNSRecordObjectType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import disk_partition_object


class PartitionListType(GeneratedsSuper):
    """The PartionListType type specifies a list of partitions."""

    subclass = None
    superclass = None
    def __init__(self, Partition=None):
        if Partition is None:
            self.Partition = []
        else:
            self.Partition = Partition
    def factory(*args_, **kwargs_):
        if PartitionListType.subclass:
            return PartitionListType.subclass(*args_, **kwargs_)
        else:
            return PartitionListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Partition(self): return self.Partition
    def set_Partition(self, Partition): self.Partition = Partition
    def add_Partition(self, value): self.Partition.append(value)
    def insert_Partition(self, index, value): self.Partition[index] = value
    def hasContent_(self):
        if (
            self.Partition
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='DiskObj:', name_='PartitionListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='PartitionListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='DiskObj:', name_='PartitionListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='DiskObj:', name_='PartitionListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Partition_ in self.Partition:
            Partition_.export(lwrite, level, 'DiskObj:', name_='Partition', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Partition':
            obj_ = disk_partition_object.DiskPartitionObjectType.factory()
            obj_.build(child_)
            self.Partition.append(obj_)
# end class PartitionListType

class DiskType(cybox_common.BaseObjectPropertyType):
    """DiskType specifies disk types, via a union of the DiskTypeEnum type
    and the atomic xs:string type. Its base type is the CybOX Core
    cybox_common.BaseObjectPropertyType, for permitting complex (i.e. regular-
    expression based) specifications.This attribute is optional and
    specifies the expected type for the value of the specified
    property."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(DiskType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.datatype = _cast(None, datatype)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if DiskType.subclass:
            return DiskType.subclass(*args_, **kwargs_)
        else:
            return DiskType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_datatype(self): return self.datatype
    def set_datatype(self, datatype): self.datatype = datatype
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(DiskType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='DiskObj:', name_='DiskType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='DiskType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='DiskObj:', name_='DiskType'):
        super(DiskType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='DiskType')
        if self.datatype is not None:

            lwrite(' datatype=%s' % (quote_attrib(self.datatype), ))
    def exportChildren(self, lwrite, level, namespace_='DiskObj:', name_='DiskType', fromsubclass_=False, pretty_print=True):
        super(DiskType, self).exportChildren(lwrite, level, 'DiskObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('datatype', node)
        if value is not None:

            self.datatype = value
        super(DiskType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class DiskType

class DiskObjectType(cybox_common.ObjectPropertiesType):
    """The DiskObjectType type is intended to characterize disk drives."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, Disk_Name=None, Disk_Size=None, Free_Space=None, Partition_List=None, Type=None):
        super(DiskObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.Disk_Name = Disk_Name
        self.Disk_Size = Disk_Size
        self.Free_Space = Free_Space
        self.Partition_List = Partition_List
        self.Type = Type
    def factory(*args_, **kwargs_):
        if DiskObjectType.subclass:
            return DiskObjectType.subclass(*args_, **kwargs_)
        else:
            return DiskObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Disk_Name(self): return self.Disk_Name
    def set_Disk_Name(self, Disk_Name): self.Disk_Name = Disk_Name
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Disk_Size(self): return self.Disk_Size
    def set_Disk_Size(self, Disk_Size): self.Disk_Size = Disk_Size
    def validate_UnsignedLongObjectPropertyType(self, value):
        # Validate type cybox_common.UnsignedLongObjectPropertyType, a restriction on None.
        pass
    def get_Free_Space(self): return self.Free_Space
    def set_Free_Space(self, Free_Space): self.Free_Space = Free_Space
    def get_Partition_List(self): return self.Partition_List
    def set_Partition_List(self, Partition_List): self.Partition_List = Partition_List
    def get_Type(self): return self.Type
    def set_Type(self, Type): self.Type = Type
    def validate_DiskType(self, value):
        # Validate type DiskType, a restriction on None.
        pass
    def hasContent_(self):
        if (
            self.Disk_Name is not None or
            self.Disk_Size is not None or
            self.Free_Space is not None or
            self.Partition_List is not None or
            self.Type is not None or
            super(DiskObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='DiskObj:', name_='DiskObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='DiskObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='DiskObj:', name_='DiskObjectType'):
        super(DiskObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='DiskObjectType')
    def exportChildren(self, lwrite, level, namespace_='DiskObj:', name_='DiskObjectType', fromsubclass_=False, pretty_print=True):
        super(DiskObjectType, self).exportChildren(lwrite, level, 'DiskObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Disk_Name is not None:
            self.Disk_Name.export(lwrite, level, 'DiskObj:', name_='Disk_Name', pretty_print=pretty_print)
        if self.Disk_Size is not None:
            self.Disk_Size.export(lwrite, level, 'DiskObj:', name_='Disk_Size', pretty_print=pretty_print)
        if self.Free_Space is not None:
            self.Free_Space.export(lwrite, level, 'DiskObj:', name_='Free_Space', pretty_print=pretty_print)
        if self.Partition_List is not None:
            self.Partition_List.export(lwrite, level, 'DiskObj:', name_='Partition_List', pretty_print=pretty_print)
        if self.Type is not None:
            self.Type.export(lwrite, level, 'DiskObj:', name_='Type', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(DiskObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Disk_Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Disk_Name(obj_)
        elif nodeName_ == 'Disk_Size':
            obj_ = cybox_common.UnsignedLongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Disk_Size(obj_)
        elif nodeName_ == 'Free_Space':
            obj_ = cybox_common.UnsignedLongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Free_Space(obj_)
        elif nodeName_ == 'Partition_List':
            obj_ = PartitionListType.factory()
            obj_.build(child_)
            self.set_Partition_List(obj_)
        elif nodeName_ == 'Type':
            obj_ = DiskType.factory()
            obj_.build(child_)
            self.set_Type(obj_)
        super(DiskObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class DiskObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': disk_partition_object.PartitionType,
    'Compilers': cybox_common.CompilersType,
    'Mount_Point': cybox_common.StringObjectPropertyType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Tool': cybox_common.ToolInformationType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Created': cybox_common.DateTimeObjectPropertyType,
    'Partition': disk_partition_object.DiskPartitionObjectType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Partition_Length': cybox_common.UnsignedLongObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Language': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Reference_Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Partition_ID': cybox_common.IntegerObjectPropertyType,
    'Free_Space': cybox_common.UnsignedLongObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Disk_Name': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Total_Space': cybox_common.UnsignedLongObjectPropertyType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Partition_Offset': cybox_common.UnsignedLongObjectPropertyType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Contributors': cybox_common.PersonnelType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Disk_Partition': disk_partition_object.DiskPartitionObjectType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'Space_Left': cybox_common.UnsignedLongObjectPropertyType,
    'Function': cybox_common.StringObjectPropertyType,
    'Disk_Size': cybox_common.UnsignedLongObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Device_Name': cybox_common.NameObjectPropertyType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Space_Used': cybox_common.UnsignedLongObjectPropertyType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Data_Size': cybox_common.DataSizeType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Disk'
        rootClass = DiskObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Disk'
        rootClass = DiskObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Disk'
        rootClass = DiskObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Disk",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "DiskObjectType",
    "PartitionListType",
    "DiskType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_event_object as win_event_binding
from cybox.objects.win_handle_object import WinHandle
from cybox.common import ObjectProperties, String


class WinEvent(ObjectProperties):
    _binding = win_event_binding
    _binding_class = win_event_binding.WindowsEventObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-event-2"
    _XSI_NS = "WinEventObj"
    _XSI_TYPE = "WindowsEventObjectType"

    name = fields.TypedField("Name", String)
    handle = fields.TypedField("Handle", WinHandle)
    type_ = fields.TypedField("Type", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_thread_object as win_thread_binding
from cybox.common import (DateTime, HexBinary, NonNegativeInteger,
        ObjectProperties, String, UnsignedInteger)
from cybox.objects.win_handle_object import WinHandle


class WinThread(ObjectProperties):
    _binding = win_thread_binding
    _binding_class = win_thread_binding.WindowsThreadObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-thread-2'
    _XSI_NS = "WinThreadObj"
    _XSI_TYPE = "WindowsThreadObjectType"

    thread_id = fields.TypedField("Thread_ID", NonNegativeInteger)
    handle = fields.TypedField("Handle", WinHandle)
    running_status = fields.TypedField("Running_Status", String)
    context = fields.TypedField("Context", String)
    priority = fields.TypedField("Priority", UnsignedInteger)
    creation_flags = fields.TypedField("Creation_Flags", HexBinary)
    creation_time = fields.TypedField("Creation_Time", DateTime)
    start_address = fields.TypedField("Start_Address", HexBinary)
    parameter_address = fields.TypedField("Parameter_Address", HexBinary)
    security_attributes = fields.TypedField("Security_Attributes", String)
    stack_size = fields.TypedField("Stack_Size", NonNegativeInteger)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import logging
import unittest

from mixbox.vendor.six import u

from cybox.core import ActionReference
from cybox.test import EntityTestCase

logger = logging.getLogger(__name__)


class TestActionReference(EntityTestCase, unittest.TestCase):
    klass = ActionReference
    _full_dict = {
        'action_id': "example:Action-1",
    }

    def test_construction(self):
        aref = ActionReference(action_id="example:Action-1")
        logger.info(aref.to_xml())
        logger.info(aref.to_dict())
        self.assertTrue(b"example:Action-1" in aref.to_xml())
        self.assertTrue("example:Action-1" in aref.to_json())


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_system_object as win_system_binding
from cybox.objects.system_object import System
from cybox.objects.win_handle_object import WinHandleList
from cybox.common import String, HexBinary


class GlobalFlag(entities.Entity):
    _binding = win_system_binding
    _binding_class = win_system_binding.GlobalFlagType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-system-2"

    abbreviation = fields.TypedField("Abbreviation", String)
    destination = fields.TypedField("Destination", String)
    hexadecimal_value = fields.TypedField("Hexadecimal_Value", HexBinary)
    symbolic_name = fields.TypedField("Symbolic_Name", String)


class GlobalFlagList(entities.EntityList):
    _binding = win_system_binding
    _binding_class = win_system_binding.GlobalFlagListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-system-2"
    global_flag = fields.TypedField("Global_Flag", GlobalFlag, multiple=True)


class WinSystem(System):
    _binding = win_system_binding
    _binding_class = win_system_binding.WindowsSystemObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/win-system-2"
    _XSI_NS = "WinSystemObj"
    _XSI_TYPE = "WindowsSystemObjectType"

    domain = fields.TypedField("Domain", String, multiple=True)
    global_flag_list = fields.TypedField("Global_Flag_List", GlobalFlagList)
    netbios_name = fields.TypedField("NetBIOS_Name", String)
    open_handle_list = fields.TypedField("Open_Handle_List", WinHandleList)
    product_id = fields.TypedField("Product_ID", String)
    product_name = fields.TypedField("Product_Name", String)
    registered_organization = fields.TypedField("Registered_Organization", String)
    registered_owner = fields.TypedField("Registered_Owner", String)
    windows_directory = fields.TypedField("Windows_Directory", String)
    windows_system_directory = fields.TypedField("Windows_System_Directory", String)
    windows_temp_directory = fields.TypedField("Windows_Temp_Directory", String)
